"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Field = void 0;
var tslib_1 = require("tslib");
var vue_1 = tslib_1.__importDefault(require("vue"));
var common_1 = require("../utils/common");
var BaseClass_1 = require("../utils/BaseClass");
var FieldExceptions_1 = require("../exceptions/FieldExceptions");
var Field = /** @class */ (function (_super) {
    tslib_1.__extends(Field, _super);
    function Field(def, fieldBind) {
        if (def === void 0) { def = {}; }
        var _this = _super.call(this) || this;
        /**
         * Field name
         */
        _this._name = null;
        /**
         * Model instance
         */
        _this._model = null;
        /**
         * Data structure when using field without model
         */
        _this._data = null;
        _this._def = def || {};
        if (fieldBind) {
            _this._name = fieldBind.name || null;
            if (Object.prototype.hasOwnProperty.call(fieldBind, 'value')) {
                if (!_this._name) {
                    _this._name = 'value';
                }
                _this._model = null;
                // Create private data structure to keep value reactive
                _this._data = vue_1.default.observable({});
                _this.value = fieldBind.value;
            }
            else {
                _this._model = fieldBind.model || null;
                _this._data = fieldBind.data ? vue_1.default.observable(fieldBind.data) : null;
            }
        }
        return _this;
    }
    /**
     * Clone field instance
     */
    Field.prototype.clone = function () {
        var FieldClass = this.cls;
        var fieldBind = {};
        this._name && (fieldBind.name = this._name);
        this._model && (fieldBind.model = this._model);
        this._data && (fieldBind.data = this._data);
        return new FieldClass(this._def, fieldBind);
    };
    /**
     * Bind field with field name and return a new instance
     */
    Field.prototype.bind = function (fieldBind) {
        var FieldClass = this.cls;
        return new FieldClass(this._def, fieldBind);
    };
    Object.defineProperty(Field.prototype, "name", {
        /**
         * Field name
         * Returns field name (Which has been set as key at fieldsDef)
         * Will throw FieldNotBoundException in case field has not been bound
         */
        get: function () {
            if (this._name === null) {
                throw new FieldExceptions_1.FieldNotBoundException(this);
            }
            return this._name;
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(Field.prototype, "attributeName", {
        /**
         * Name of attribute in data
         */
        get: function () {
            return this.definition.attributeName || this.name;
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(Field.prototype, "definition", {
        /**
         * Field definition
         */
        get: function () {
            return this._def;
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(Field.prototype, "model", {
        /**
         * Assigned model
         * Will throw FieldNotBoundException in case field has not been bound to a model
         */
        get: function () {
            if (this._model === null) {
                throw new FieldExceptions_1.FieldNotBoundException(this);
            }
            return this._model;
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(Field.prototype, "data", {
        /**
         * Return bound data or data from bound model
         * Will throw FieldNotBoundException if field is not bound to data or model
         */
        get: function () {
            return this._data || this.model.data;
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(Field.prototype, "value", {
        /**
         * Property mapper for getValue
         */
        get: function () {
            return this.getValue();
        },
        /**
         * Property mapper for setValue
         */
        set: function (value) {
            this.setValue(value);
        },
        enumerable: false,
        configurable: true
    });
    /**
     * Field value
     * Returns async field value from data by calling valueGetter with data
     */
    Field.prototype.getValue = function () {
        return tslib_1.__awaiter(this, void 0, void 0, function () {
            return tslib_1.__generator(this, function (_a) {
                return [2 /*return*/, this.valueGetter(this.data)];
            });
        });
    };
    /**
     * Field value setter
     * Sets field value to model data by calling valueSetter with data
     */
    Field.prototype.setValue = function (value) {
        this.valueSetter(value, this.data);
    };
    /**
     * Parses raw value with valueParser and sets field value by calling valueSetter
     */
    Field.prototype.setParseValue = function (rawValue) {
        return tslib_1.__awaiter(this, void 0, void 0, function () {
            var parsedValue;
            return tslib_1.__generator(this, function (_a) {
                switch (_a.label) {
                    case 0: return [4 /*yield*/, this.valueParser(rawValue)];
                    case 1:
                        parsedValue = _a.sent();
                        this.value = parsedValue;
                        return [2 /*return*/, parsedValue];
                }
            });
        });
    };
    Object.defineProperty(Field.prototype, "label", {
        /**
         * Field label
         */
        get: function () {
            return common_1.promiseEval(this.definition.label, this);
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(Field.prototype, "options", {
        /**
         * Returns async field options with validation and default values depending on field type
         */
        get: function () {
            var _this = this;
            return common_1.promiseEval(this.definition.options, this).then(function (options) { return _this.validateOptions(options); });
        },
        enumerable: false,
        configurable: true
    });
    /**
     * Validate field options and set default values depending on field type
     */
    Field.prototype.validateOptions = function (options) {
        return tslib_1.__awaiter(this, void 0, void 0, function () {
            return tslib_1.__generator(this, function (_a) {
                return [2 /*return*/, options || {}];
            });
        });
    };
    Object.defineProperty(Field.prototype, "isPrimaryKey", {
        /**
         * Returns boolean whether field is a primary key
         */
        get: function () {
            return Boolean(this.definition.primaryKey);
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(Field.prototype, "isNestedAttribute", {
        /**
         * Returns boolean whether attribute is an nested data structure
         */
        get: function () {
            return this.attributeName.includes('.');
        },
        enumerable: false,
        configurable: true
    });
    /**
     * Retrieve value from data structure according to attributeName
     * Uses nested syntax from attributeName (e.g. "address.city" -> {address: {city: 'New York'}})
     * Will return null if value is not available
     */
    Field.prototype.valueGetter = function (data) {
        if (!data || typeof data !== 'object')
            return null;
        // No nested attribute name
        if (!this.isNestedAttribute) {
            var value = data[this.attributeName];
            return !common_1.isNull(value) ? value : null;
        }
        // Attribute name contains nested attributes e.g. obj.nested.field
        var subFields = this.attributeName.split('.');
        var currentObject = data;
        var subFieldName;
        for (var _i = 0, subFields_1 = subFields; _i < subFields_1.length; _i++) {
            subFieldName = subFields_1[_i];
            currentObject = currentObject[subFieldName];
            if (common_1.isNull(currentObject)) {
                return null;
            }
        }
        /* istanbul ignore else */
        if (!common_1.isNull(currentObject)) {
            return currentObject;
        }
        else {
            return null;
        }
    };
    /**
     * Set value of field to data by using attributeName
     * Will create nested structure from attributeName (e.g. "address.city" -> {address: {city: 'New York'}})
     */
    Field.prototype.valueSetter = function (value, data) {
        if (!this.attributeName.includes('.')) {
            vue_1.default.set(data, this.attributeName, value);
        }
        else {
            var subFields = this.attributeName.split('.');
            // Retrieve start position for set
            var fieldIndex = 0;
            var subFieldName = subFields[fieldIndex];
            var currentData = data;
            while (true) {
                if (Object.prototype.hasOwnProperty.call(currentData, subFieldName) && !common_1.isNull(currentData[subFieldName])) {
                    currentData = currentData[subFieldName];
                }
                else {
                    break;
                }
                fieldIndex++;
                subFieldName = subFields[fieldIndex];
                if (fieldIndex + 1 === subFields.length) {
                    break;
                }
            }
            // Build nested object which will be set as value
            var setValue = subFields.splice(fieldIndex + 1).reduceRight(function (obj, subFieldName) {
                var newObj = {};
                newObj[subFieldName] = obj;
                return newObj;
            }, value);
            vue_1.default.set(currentData, subFieldName, setValue);
        }
    };
    /**
     * Parse a raw value and return the parsed value with valid data type
     */
    Field.prototype.valueParser = function (rawValue) {
        return tslib_1.__awaiter(this, void 0, void 0, function () {
            return tslib_1.__generator(this, function (_a) {
                return [2 /*return*/, rawValue];
            });
        });
    };
    /**
     * Map value from a data structure to another data structure
     */
    Field.prototype.mapFieldValue = function (fromData, toData) {
        var value = this.valueGetter(fromData);
        this.valueSetter(value, toData);
    };
    return Field;
}(BaseClass_1.BaseClass));
exports.Field = Field;
