"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.ModelManager = void 0;
var tslib_1 = require("tslib");
var axios_1 = tslib_1.__importDefault(require("axios"));
var ConfigHandler_1 = require("../utils/ConfigHandler");
var APIExceptions_1 = require("../exceptions/APIExceptions");
/**
 * ModelManager
 * Provides interface for model to retrieve data from backend api
 */
var ModelManager = /** @class */ (function () {
    function ModelManager(model) {
        this.model = model;
    }
    /**
     * Fill ServiceStoreOptions default options from params
     */
    ModelManager.prototype.getServiceStoreOptions = function (options, params) {
        return tslib_1.__assign({ noCache: Boolean(params && params.noCache), noRequestAggregation: Boolean(params && params.noRequestAggregation), refreshCache: Boolean(params && params.refreshCache) }, options);
    };
    /**
     * Retrieve specific model instance from service
     */
    ModelManager.prototype.detail = function (pk, params) {
        return tslib_1.__awaiter(this, void 0, void 0, function () {
            var parents, Model, data;
            return tslib_1.__generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        parents = params && params.parents;
                        Model = this.model;
                        return [4 /*yield*/, this.retrieveDetailData(pk, params)];
                    case 1:
                        data = _a.sent();
                        return [2 /*return*/, new Model(data, parents)];
                }
            });
        });
    };
    /**
     * Retrieve list of model instances from service
     */
    ModelManager.prototype.list = function (params) {
        return tslib_1.__awaiter(this, void 0, void 0, function () {
            var _params, Model, url, keyBuilder, options, dataList;
            return tslib_1.__generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        _params = tslib_1.__assign({}, (params || {}));
                        // Disable cache of list requests by default
                        if (!Object.prototype.hasOwnProperty.call(_params, 'noCache')) {
                            _params.noCache = true;
                        }
                        Model = this.model;
                        return [4 /*yield*/, Model.getListUrl(_params.parents)
                            // Build key for request by url and filter
                        ];
                    case 1:
                        url = _a.sent();
                        keyBuilder = [url];
                        if (_params.filter && Object.keys(_params.filter).length > 0) {
                            keyBuilder.push(JSON.stringify(_params.filter));
                        }
                        options = this.getServiceStoreOptions({
                            key: keyBuilder.join('?'),
                            sendRequest: this.sendListRequest.bind(this),
                            args: [url, _params]
                        }, _params);
                        return [4 /*yield*/, Model.store.getData(options)];
                    case 2:
                        dataList = _a.sent();
                        return [2 /*return*/, dataList.map(function (data) { return new Model(data, _params.parents); })];
                }
            });
        });
    };
    /**
     * Create single instance
     */
    ModelManager.prototype.create = function (data, params) {
        return tslib_1.__awaiter(this, void 0, void 0, function () {
            var parents, Model, url;
            return tslib_1.__generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        parents = params && params.parents;
                        Model = this.model;
                        return [4 /*yield*/, Model.getListUrl(parents)];
                    case 1:
                        url = _a.sent();
                        return [2 /*return*/, this.sendCreateRequest(url, data, params)];
                }
            });
        });
    };
    /**
     * Update single instance
     */
    ModelManager.prototype.update = function (pk, data, params) {
        return tslib_1.__awaiter(this, void 0, void 0, function () {
            var parents, Model, url;
            return tslib_1.__generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        parents = params && params.parents;
                        Model = this.model;
                        return [4 /*yield*/, Model.getDetailUrl(pk, parents)];
                    case 1:
                        url = _a.sent();
                        if (params && params.partial) {
                            return [2 /*return*/, this.sendPartialUpdateRequest(url, pk, data, params)];
                        }
                        else {
                            return [2 /*return*/, this.sendUpdateRequest(url, pk, data, params)];
                        }
                        return [2 /*return*/];
                }
            });
        });
    };
    /**
     * Delete single instance
     */
    ModelManager.prototype.delete = function (pk, params) {
        return tslib_1.__awaiter(this, void 0, void 0, function () {
            var parents, Model, url;
            return tslib_1.__generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        parents = params && params.parents;
                        Model = this.model;
                        return [4 /*yield*/, Model.getDetailUrl(pk, parents)];
                    case 1:
                        url = _a.sent();
                        return [2 /*return*/, this.sendDeleteRequest(url, pk, params)];
                }
            });
        });
    };
    /**
     * Build config for axios retrieve request
     */
    ModelManager.prototype.buildRetrieveRequestConfig = function (params) {
        return tslib_1.__awaiter(this, void 0, void 0, function () {
            var config;
            return tslib_1.__generator(this, function (_a) {
                if (!params)
                    return [2 /*return*/, {}];
                config = {};
                if (params.filter && Object.keys(params.filter).length) {
                    config.params = params.filter;
                }
                return [2 /*return*/, config];
            });
        });
    };
    /**
     * Retrieve detail data from ServiceStore
     */
    ModelManager.prototype.retrieveDetailData = function (pk, params) {
        return tslib_1.__awaiter(this, void 0, void 0, function () {
            var parents, Model, url, options;
            return tslib_1.__generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        parents = params && params.parents;
                        Model = this.model;
                        return [4 /*yield*/, Model.getDetailUrl(pk, parents)];
                    case 1:
                        url = _a.sent();
                        options = this.getServiceStoreOptions({
                            key: url,
                            sendRequest: this.sendDetailRequest.bind(this),
                            args: [url, pk, params]
                        }, params);
                        return [2 /*return*/, Model.store.getData(options)];
                }
            });
        });
    };
    /**
     * Send actual detail service request and map data before caching
     */
    ModelManager.prototype.sendDetailRequest = function (options, url, pk, params) {
        return tslib_1.__awaiter(this, void 0, void 0, function () {
            var requestConfig, response, error_1;
            return tslib_1.__generator(this, function (_a) {
                switch (_a.label) {
                    case 0: return [4 /*yield*/, this.buildRetrieveRequestConfig(params)];
                    case 1:
                        requestConfig = _a.sent();
                        ConfigHandler_1.configHandler.emitEvent('onSendDetailRequest', [{ modelManager: this, url: url, pk: pk, params: params }]);
                        _a.label = 2;
                    case 2:
                        _a.trys.push([2, 4, , 6]);
                        return [4 /*yield*/, axios_1.default.get(url, requestConfig)];
                    case 3:
                        response = _a.sent();
                        return [3 /*break*/, 6];
                    case 4:
                        error_1 = _a.sent();
                        return [4 /*yield*/, this.handleResponseError(error_1)];
                    case 5: throw _a.sent();
                    case 6: return [2 /*return*/, this.mapDetailResponseBeforeCache(options, response.data, url, pk, params)];
                }
            });
        });
    };
    /**
     * Map raw response data from detail service request before cache
     */
    ModelManager.prototype.mapDetailResponseBeforeCache = function (options, data, url, pk, params) {
        return tslib_1.__awaiter(this, void 0, void 0, function () {
            return tslib_1.__generator(this, function (_a) {
                return [2 /*return*/, data];
            });
        });
    };
    /**
     * Send actual list service request and map data before caching
     */
    ModelManager.prototype.sendListRequest = function (options, url, params) {
        return tslib_1.__awaiter(this, void 0, void 0, function () {
            var requestConfig, response, error_2;
            return tslib_1.__generator(this, function (_a) {
                switch (_a.label) {
                    case 0: return [4 /*yield*/, this.buildRetrieveRequestConfig(params)];
                    case 1:
                        requestConfig = _a.sent();
                        ConfigHandler_1.configHandler.emitEvent('onSendListRequest', [{ modelManager: this, url: url, params: params }]);
                        _a.label = 2;
                    case 2:
                        _a.trys.push([2, 4, , 6]);
                        return [4 /*yield*/, axios_1.default.get(url, requestConfig)];
                    case 3:
                        response = _a.sent();
                        return [3 /*break*/, 6];
                    case 4:
                        error_2 = _a.sent();
                        return [4 /*yield*/, this.handleResponseError(error_2)];
                    case 5: throw _a.sent();
                    case 6: return [2 /*return*/, this.mapListResponseBeforeCache(options, response.data, url, params)];
                }
            });
        });
    };
    /**
     * Map raw response data from list service request before cache
     */
    ModelManager.prototype.mapListResponseBeforeCache = function (options, data, url, params) {
        return tslib_1.__awaiter(this, void 0, void 0, function () {
            return tslib_1.__generator(this, function (_a) {
                return [2 /*return*/, data];
            });
        });
    };
    /**
     * Send actual create (POST) service request
     */
    ModelManager.prototype.sendCreateRequest = function (url, data, params) {
        return tslib_1.__awaiter(this, void 0, void 0, function () {
            var response, error_3;
            return tslib_1.__generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        ConfigHandler_1.configHandler.emitEvent('onSendCreateRequest', [{ modelManager: this, url: url, data: data, params: params }]);
                        _a.label = 1;
                    case 1:
                        _a.trys.push([1, 3, , 5]);
                        return [4 /*yield*/, axios_1.default.post(url, data)];
                    case 2:
                        response = _a.sent();
                        return [3 /*break*/, 5];
                    case 3:
                        error_3 = _a.sent();
                        return [4 /*yield*/, this.handleResponseError(error_3)];
                    case 4: throw _a.sent();
                    case 5: return [2 /*return*/, response.data];
                }
            });
        });
    };
    /**
     * Send actual update (PUT) service request
     */
    ModelManager.prototype.sendUpdateRequest = function (url, pk, data, params) {
        return tslib_1.__awaiter(this, void 0, void 0, function () {
            var response, error_4;
            return tslib_1.__generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        ConfigHandler_1.configHandler.emitEvent('onSendUpdateRequest', [{ modelManager: this, url: url, pk: pk, data: data, params: params }]);
                        _a.label = 1;
                    case 1:
                        _a.trys.push([1, 3, , 5]);
                        return [4 /*yield*/, axios_1.default.put(url, data)];
                    case 2:
                        response = _a.sent();
                        return [3 /*break*/, 5];
                    case 3:
                        error_4 = _a.sent();
                        return [4 /*yield*/, this.handleResponseError(error_4)];
                    case 4: throw _a.sent();
                    case 5: return [2 /*return*/, response.data];
                }
            });
        });
    };
    /**
     * Send actual partial update (PATCH) service request
     */
    ModelManager.prototype.sendPartialUpdateRequest = function (url, pk, data, params) {
        return tslib_1.__awaiter(this, void 0, void 0, function () {
            var response, error_5;
            return tslib_1.__generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        ConfigHandler_1.configHandler.emitEvent('onSendPartialUpdateRequest', [{ modelManager: this, url: url, pk: pk, data: data, params: params }]);
                        _a.label = 1;
                    case 1:
                        _a.trys.push([1, 3, , 5]);
                        return [4 /*yield*/, axios_1.default.patch(url, data)];
                    case 2:
                        response = _a.sent();
                        return [3 /*break*/, 5];
                    case 3:
                        error_5 = _a.sent();
                        return [4 /*yield*/, this.handleResponseError(error_5)];
                    case 4: throw _a.sent();
                    case 5: return [2 /*return*/, response.data];
                }
            });
        });
    };
    /**
     * Send actual delete (DELETE) service request
     */
    ModelManager.prototype.sendDeleteRequest = function (url, pk, params) {
        return tslib_1.__awaiter(this, void 0, void 0, function () {
            var error_6;
            return tslib_1.__generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        ConfigHandler_1.configHandler.emitEvent('onSendDeleteRequest', [{ modelManager: this, url: url, pk: pk, params: params }]);
                        _a.label = 1;
                    case 1:
                        _a.trys.push([1, 3, , 5]);
                        return [4 /*yield*/, axios_1.default.delete(url)];
                    case 2:
                        _a.sent();
                        return [3 /*break*/, 5];
                    case 3:
                        error_6 = _a.sent();
                        return [4 /*yield*/, this.handleResponseError(error_6)];
                    case 4: throw _a.sent();
                    case 5: return [2 /*return*/, null];
                }
            });
        });
    };
    /**
     * Receive error from service and map to api exceptions
     */
    ModelManager.prototype.handleResponseError = function (error) {
        return tslib_1.__awaiter(this, void 0, void 0, function () {
            return tslib_1.__generator(this, function (_a) {
                ConfigHandler_1.configHandler.emitEvent('onResponseError', [{ modelManager: this, error: error }]);
                if (error.response) {
                    switch (error.response.status) {
                        case APIExceptions_1.BadRequestAPIException.statusCode:
                            return [2 /*return*/, new APIExceptions_1.BadRequestAPIException(error.response)];
                        case APIExceptions_1.UnauthorizedAPIException.statusCode:
                            return [2 /*return*/, new APIExceptions_1.UnauthorizedAPIException(error.response)];
                        case APIExceptions_1.ForbiddenAPIException.statusCode:
                            return [2 /*return*/, new APIExceptions_1.ForbiddenAPIException(error.response)];
                        case APIExceptions_1.NotFoundAPIException.statusCode:
                            return [2 /*return*/, new APIExceptions_1.NotFoundAPIException(error.response)];
                        case APIExceptions_1.InternalServerErrorAPIException.statusCode:
                            return [2 /*return*/, new APIExceptions_1.InternalServerErrorAPIException(error.response)];
                        default:
                            return [2 /*return*/, new APIExceptions_1.APIException(error.response)];
                    }
                }
                else {
                    return [2 /*return*/, error];
                }
                return [2 /*return*/];
            });
        });
    };
    return ModelManager;
}());
exports.ModelManager = ModelManager;
