"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.ServiceModel = void 0;
var tslib_1 = require("tslib");
var BaseModel_1 = require("./BaseModel");
var ModelManager_1 = require("./ModelManager");
var common_1 = require("../utils/common");
var ServiceStore_1 = require("../store/ServiceStore");
var ModelExceptions_1 = require("../exceptions/ModelExceptions");
/**
 * ServiceModel
 * Model with service interface to retrieve data from backend api
 */
var ServiceModel = /** @class */ (function (_super) {
    tslib_1.__extends(ServiceModel, _super);
    /**
     * Constructor
     */
    function ServiceModel(data, parents) {
        if (data === void 0) { data = {}; }
        if (parents === void 0) { parents = {}; }
        var _this = _super.call(this, data) || this;
        _this._parents = common_1.clone(parents);
        return _this;
    }
    Object.defineProperty(ServiceModel, "store", {
        /**
         * Getter to simulate static class property with fixed inheritance
         */
        get: function () {
            // Check whether model has property __modelRegistered and not inherited from super class
            if (!Object.prototype.hasOwnProperty.call(this, '__store')) {
                this.__store = this.createStoreModule();
            }
            return this.__store;
        },
        enumerable: false,
        configurable: true
    });
    /**
     * Function to return list url of model according to parents
     */
    ServiceModel.getListUrl = function (parents) {
        return tslib_1.__awaiter(this, void 0, void 0, function () {
            var url;
            var _this = this;
            return tslib_1.__generator(this, function (_a) {
                this.checkServiceParents(parents);
                url = (function () {
                    if (typeof _this.urls === 'string') {
                        return _this.urls;
                    }
                    else if (_this.urls.LIST) {
                        return _this.urls.LIST;
                    }
                    else if (_this.urls.BASE) {
                        return _this.urls.BASE;
                    }
                    else {
                        throw new ModelExceptions_1.MissingUrlException(_this.name);
                    }
                })();
                if (parents) {
                    return [2 /*return*/, common_1.format(url, parents)];
                }
                else {
                    return [2 /*return*/, url];
                }
                return [2 /*return*/];
            });
        });
    };
    /**
     * Function to return detail url of model according to parents
     */
    ServiceModel.getDetailUrl = function (pk, parents) {
        return tslib_1.__awaiter(this, void 0, void 0, function () {
            var url;
            var _this = this;
            return tslib_1.__generator(this, function (_a) {
                this.checkServiceParents(parents);
                url = (function () {
                    if (typeof _this.urls === 'string') {
                        return _this.urls + '{pk}/';
                    }
                    else if (_this.urls.DETAIL) {
                        return _this.urls.DETAIL;
                    }
                    else if (_this.urls.BASE) {
                        return _this.urls.BASE + '{pk}/';
                    }
                    else {
                        throw new ModelExceptions_1.MissingUrlException(_this.name);
                    }
                })();
                return [2 /*return*/, common_1.format(url, tslib_1.__assign({ pk: pk }, parents || {}))];
            });
        });
    };
    Object.defineProperty(ServiceModel, "objects", {
        /**
         * Retrieve instance of ModelManager
         */
        get: function () {
            if (!Object.prototype.hasOwnProperty.call(this, '__modelManager')) {
                this.register();
                var ServiceClass = this.ModelManager;
                this.__modelManager = new ServiceClass(this);
            }
            return this.__modelManager;
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(ServiceModel.prototype, "parents", {
        /**
         * Return model parents
         */
        get: function () {
            return this._parents;
        },
        /**
         * Set deep copy of model parents to avoid unwanted mutations
         */
        set: function (parents) {
            this._parents = common_1.clone(parents);
        },
        enumerable: false,
        configurable: true
    });
    /**
     * Check whether all required parent values have been given
     * @param parents
     */
    ServiceModel.checkServiceParents = function (parents) {
        var _parents = parents || {};
        if (this.parentNames.length < Object.keys(_parents).length) {
            console.error('[vue-service-model] Too much parents given', this.name, _parents);
            return false;
        }
        else if (this.parentNames.length > 0) {
            var missingParents = this.parentNames.filter(function (name) { return !_parents[name]; });
            if (missingParents.length) {
                console.error('[vue-service-model] Missing parents', this.name, missingParents);
                return false;
            }
        }
        return true;
    };
    /**
     * Map data by field names for partial update
     */
    ServiceModel.prototype.mapPartialUpdateFields = function (data, updateFields) {
        return tslib_1.__awaiter(this, void 0, void 0, function () {
            var updateData;
            var _this = this;
            return tslib_1.__generator(this, function (_a) {
                updateData = {};
                updateFields.forEach(function (fieldName) {
                    var field = _this.getField(fieldName);
                    field.mapFieldValue(data, updateData);
                });
                return [2 /*return*/, updateData];
            });
        });
    };
    /**
     * Reload model data from service. Overwrites changes made to model data
     * Returns true if successful
     */
    ServiceModel.prototype.reload = function () {
        return tslib_1.__awaiter(this, void 0, void 0, function () {
            var pk, cls, _a;
            return tslib_1.__generator(this, function (_b) {
                switch (_b.label) {
                    case 0:
                        pk = this.pk;
                        if (pk === null)
                            return [2 /*return*/, false];
                        cls = this.constructor;
                        _a = this;
                        return [4 /*yield*/, cls.objects.retrieveDetailData(pk, {
                                refreshCache: true,
                                parents: this.parents
                            })];
                    case 1:
                        _a.data = _b.sent();
                        return [2 /*return*/, true];
                }
            });
        });
    };
    /**
     * Call either .create() or .update() by checking whether primary key is set or not
     * Returns true if create has been called
     */
    ServiceModel.prototype.save = function () {
        return tslib_1.__awaiter(this, void 0, void 0, function () {
            return tslib_1.__generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        if (!(this.pk === null)) return [3 /*break*/, 2];
                        return [4 /*yield*/, this.create()];
                    case 1:
                        _a.sent();
                        return [2 /*return*/, true];
                    case 2: return [4 /*yield*/, this.update()];
                    case 3:
                        _a.sent();
                        return [2 /*return*/, false];
                }
            });
        });
    };
    /**
     * Create current model instance by calling objects.create()
     * Updates model data from response if set
     * Returns true if successful
     */
    ServiceModel.prototype.create = function () {
        return tslib_1.__awaiter(this, void 0, void 0, function () {
            var cls, data;
            return tslib_1.__generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        cls = this.constructor;
                        return [4 /*yield*/, cls.objects.create(this.data, {
                                parents: this.parents
                            })];
                    case 1:
                        data = _a.sent();
                        if (data && Object.keys(data).length) {
                            this.data = data;
                        }
                        return [2 /*return*/, true];
                }
            });
        });
    };
    /**
     * Update current model instance by calling objects.update()
     * Updates model data from response if set
     * Returns true if successful
     */
    ServiceModel.prototype.update = function (options) {
        return tslib_1.__awaiter(this, void 0, void 0, function () {
            var pk, updateFields, partial, updateData, _a, cls, data;
            return tslib_1.__generator(this, function (_b) {
                switch (_b.label) {
                    case 0:
                        pk = this.pk;
                        if (pk === null)
                            return [2 /*return*/, false];
                        updateFields = options && options.updateFields;
                        partial = Boolean(updateFields);
                        if (!partial) return [3 /*break*/, 2];
                        return [4 /*yield*/, this.mapPartialUpdateFields(this.data, updateFields)];
                    case 1:
                        _a = _b.sent();
                        return [3 /*break*/, 3];
                    case 2:
                        _a = this.data;
                        _b.label = 3;
                    case 3:
                        updateData = _a;
                        cls = this.constructor;
                        return [4 /*yield*/, cls.objects.update(pk, updateData, {
                                parents: this.parents,
                                partial: partial
                            })];
                    case 4:
                        data = _b.sent();
                        if (data && Object.keys(data).length) {
                            this.data = data;
                        }
                        return [2 /*return*/, true];
                }
            });
        });
    };
    /**
     * Delete current model instance by calling objects.delete()
     * Returns true if successful
     */
    ServiceModel.prototype.delete = function () {
        return tslib_1.__awaiter(this, void 0, void 0, function () {
            var pk, cls;
            return tslib_1.__generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        pk = this.pk;
                        if (pk === null)
                            return [2 /*return*/, false];
                        cls = this.constructor;
                        return [4 /*yield*/, cls.objects.delete(pk, {
                                parents: this.parents
                            })];
                    case 1:
                        _a.sent();
                        return [2 /*return*/, true];
                }
            });
        });
    };
    /**
     * Create instance of store class
     */
    ServiceModel.createStoreModule = function () {
        var ServiceStoreClass = this.storeClass;
        return new ServiceStoreClass(this.cacheDuration);
    };
    /**
     * Default URL definition for backend APIs
     * Fill either LIST/DETAIL or BASE url or use other urls by overwriting getListUrl/getDetailUrl
     */
    ServiceModel.urls = {};
    /**
     * List of parent names to be used in url
     */
    ServiceModel.parentNames = [];
    /**
     * Duration to cache requested data in seconds. 0: no cache. null: Cache forever
     */
    ServiceModel.cacheDuration = 30;
    /**
     * Cache store class to use
     */
    ServiceModel.storeClass = ServiceStore_1.ServiceStore;
    /**
     * Manager class of model
     */
    ServiceModel.ModelManager = ModelManager_1.ModelManager;
    return ServiceModel;
}(BaseModel_1.BaseModel));
exports.ServiceModel = ServiceModel;
