"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.ServiceStore = void 0;
var tslib_1 = require("tslib");
var ServiceStore = /** @class */ (function () {
    function ServiceStore(cacheDuration) {
        this.cacheDuration = cacheDuration;
        this._data = {};
        this._requests = {};
    }
    /**
     * Get data of options.key either from cache or by calling options.sendRequest
     * @param options
     */
    ServiceStore.prototype.getData = function (options) {
        return tslib_1.__awaiter(this, void 0, void 0, function () {
            var key, useCache, data;
            return tslib_1.__generator(this, function (_a) {
                key = options.key;
                useCache = this.useCache(options);
                // Retrieve data from cache
                if (useCache && Object.prototype.hasOwnProperty.call(this._data, key)) {
                    data = this._data[key];
                    // Cache expired
                    if (!data.expires || data.expires > Date.now()) {
                        return [2 /*return*/, data.data];
                    }
                }
                return [2 /*return*/, this.loadData(options)];
            });
        });
    };
    /**
     * Loads data by calling options.sendRequest. If request of same key has already started return attach to this request
     * @param options
     */
    ServiceStore.prototype.loadData = function (options) {
        return tslib_1.__awaiter(this, void 0, void 0, function () {
            var key, aggregatedRequest, request;
            var _this = this;
            return tslib_1.__generator(this, function (_a) {
                key = options.key;
                aggregatedRequest = this.getRequestAggregation(options);
                if (aggregatedRequest)
                    return [2 /*return*/, aggregatedRequest
                        // Actual request and save result in cache
                    ];
                request = options.sendRequest.apply(options, tslib_1.__spreadArrays([options], (options.args || []))).then(function (data) {
                    if (!options.noCache) {
                        _this._setData(key, data);
                    }
                    _this.removeRequest(key);
                    return data;
                }, function (error) {
                    _this.removeRequest(key);
                    throw error;
                });
                if (!options.noRequestAggregation) {
                    this._setRequest(key, request);
                }
                return [2 /*return*/, request];
            });
        });
    };
    /**
     * Save data in cache if required
     * @param key
     * @param data
     */
    ServiceStore.prototype._setData = function (key, data) {
        if (this.cacheDuration !== 0) {
            var _data = { data: data };
            if (this.cacheDuration !== null) {
                _data.expires = Date.now() + (this.cacheDuration * 1000);
            }
            this._data[key] = _data;
        }
    };
    /**
     * Save started request promise
     * @param key
     * @param request
     */
    ServiceStore.prototype._setRequest = function (key, request) {
        this._requests[key] = request;
    };
    /**
     * Check whether data should be retrieved from cache or not
     * @param options
     */
    ServiceStore.prototype.useCache = function (options) {
        return !options.noCache && !options.refreshCache;
    };
    /**
     * Check whether request is already in queue and return request promise if so
     * @param options
     */
    ServiceStore.prototype.getRequestAggregation = function (options) {
        if (options.noRequestAggregation)
            return null;
        var key = options.key;
        if (Object.prototype.hasOwnProperty.call(this._requests, key)) {
            return this._requests[key];
        }
        return null;
    };
    /**
     * Remove request promise
     * @param key
     */
    ServiceStore.prototype.removeRequest = function (key) {
        delete this._requests[key];
    };
    /**
     * Clean up data and remove expired cache
     */
    ServiceStore.prototype.clean = function () {
        var key;
        var expiredTime = Date.now();
        for (var _i = 0, _a = Object.keys(this._data); _i < _a.length; _i++) {
            key = _a[_i];
            var expires = this._data[key].expires;
            if (expires && expires < expiredTime) {
                delete this._data[key];
            }
        }
    };
    /**
     * clear complete cache
     */
    ServiceStore.prototype.clear = function () {
        this._data = {};
        this._requests = {};
    };
    return ServiceStore;
}());
exports.ServiceStore = ServiceStore;
