"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.NO_VALUE = exports.format = exports.promiseEval = exports.funcEval = exports.deepCompare = exports.mergeDeep = exports.clone = exports.isNull = exports.isObject = void 0;
var tslib_1 = require("tslib");
/**
 * Simple object check.
 * Source: https://stackoverflow.com/questions/27936772/how-to-deep-merge-instead-of-shallow-merge
 * @param {Any} item
 * @param {Boolean} arrayIsObject
 * @returns {Boolean}
 */
exports.isObject = function (item, arrayIsObject) {
    if (arrayIsObject === void 0) { arrayIsObject = false; }
    if (item && typeof item === 'object') {
        return !Array.isArray(item) || arrayIsObject;
    }
    else {
        return false;
    }
};
/**
 * Check whether value is null or undefined is empty
 * @param {Any} value
 * @returns {Boolean}
 */
exports.isNull = function (value) {
    return value === null || value === undefined;
};
/**
 * clone
 * deep clone javascript object
 * @param {Any} obj
 * @returns {Any}
 */
exports.clone = function (obj) {
    if (obj === null || typeof obj !== 'object') {
        return obj;
    }
    var copy = obj.constructor();
    for (var _i = 0, _a = Object.keys(obj); _i < _a.length; _i++) {
        var attr = _a[_i];
        copy[attr] = exports.clone(obj[attr]);
    }
    return copy;
};
/**
 * Deep merge two objects.
 * Source: https://stackoverflow.com/questions/27936772/how-to-deep-merge-instead-of-shallow-merge
 * @param {Object} target
 * @param {Object} sources
 * @returns {Object}
 */
exports.mergeDeep = function (target) {
    var _a, _b;
    var sources = [];
    for (var _i = 1; _i < arguments.length; _i++) {
        sources[_i - 1] = arguments[_i];
    }
    if (!sources.length)
        return target;
    var source = sources.shift();
    /* istanbul ignore else */
    if (source && exports.isObject(target) && exports.isObject(source)) {
        for (var _c = 0, _d = Object.keys(source); _c < _d.length; _c++) {
            var key = _d[_c];
            var value = source[key];
            if (exports.isObject(value)) {
                if (!target[key])
                    Object.assign(target, (_a = {}, _a[key] = {}, _a));
                exports.mergeDeep(target[key], value);
            }
            else {
                Object.assign(target, (_b = {}, _b[key] = value, _b));
            }
        }
    }
    return exports.mergeDeep.apply(void 0, tslib_1.__spreadArrays([target], sources));
};
/**
 * Deep compare two objects
 * Will not work with circular objects and only compares method names
 * @param {Any} obj1
 * @param {Any} obj2
 * @returns {Boolean}
 */
exports.deepCompare = function (obj1, obj2) {
    if (obj1 === obj2)
        return true;
    if (exports.isNull(obj1) || exports.isNull(obj2))
        return false;
    if (Array.isArray(obj1) !== Array.isArray(obj2))
        return false;
    if (exports.isObject(obj1, true) !== exports.isObject(obj2, true))
        return false;
    if (!exports.isObject(obj1, true)) {
        // Primitive objects! -> Simple compare with: ===
        return obj1 === obj2;
    }
    if (Object.keys(obj1).length !== Object.keys(obj2).length)
        return false;
    for (var _i = 0, _a = Object.keys(obj1); _i < _a.length; _i++) {
        var p = _a[_i];
        if (!Object.prototype.hasOwnProperty.call(obj2, p)) {
            return false;
        }
        switch (typeof (obj1[p])) {
            case 'object':
                if (!exports.deepCompare(obj1[p], obj2[p])) {
                    return false;
                }
                break;
            case 'function':
                if (typeof (obj2[p]) === 'undefined' || (obj1[p].toString() !== obj2[p].toString())) {
                    return false;
                }
                break;
            default:
                if (obj1[p] !== obj2[p]) {
                    return false;
                }
        }
    }
    return true;
};
/**
 * Call value if callable else return value
 * @param {Any} value
 * @param {any} context
 * @param {Any} args
 * @returns {Any}
 */
exports.funcEval = function (value, context) {
    var args = [];
    for (var _i = 2; _i < arguments.length; _i++) {
        args[_i - 2] = arguments[_i];
    }
    return typeof value === 'function' ? value.call.apply(value, tslib_1.__spreadArrays([context], args)) : value;
};
/**
 * Call value if callable else use value to resolve Promise
 * @param {Any} value
 * @param {Any} context
 * @param {Any} args
 * @returns {Promise}
 */
exports.promiseEval = function (value, context) {
    var args = [];
    for (var _i = 2; _i < arguments.length; _i++) {
        args[_i - 2] = arguments[_i];
    }
    return Promise.resolve(exports.funcEval.apply(void 0, tslib_1.__spreadArrays([value, context], args)));
};
/**
 * Format string with placeholders
 * E.g. "Test String {value}" + {value: 5} = "Test String 5"
 * https://stackoverflow.com/questions/610406/javascript-equivalent-to-printf-string-format/18234317
 */
exports.format = function (str, args) {
    return Object.keys(args).reduce(function (_str, key) {
        return _str.replace(new RegExp("\\{" + key + "\\}", 'gi'), args[key]);
    }, str.toString());
};
exports.NO_VALUE = {};
